@@ -1,8 +1,8 @@
 ets <- function(y, model="ZZZ", damped=NULL,
-    alpha=NULL, beta=NULL, gamma=NULL, phi=NULL, additive.only=FALSE, lambda=NULL,
+    alpha=NULL, beta=NULL, gamma=NULL, phi=NULL, additive.only=FALSE, lambda=NULL, biasadj=FALSE,
     lower=c(rep(0.0001,3), 0.8), upper=c(rep(0.9999,3),0.98),
     opt.crit=c("lik","amse","mse","sigma","mae"), nmse=3, bounds=c("both","usual","admissible"),
-    ic=c("aicc","aic","bic"),restrict=TRUE, allow.multiplicative.trend=FALSE, 
+    ic=c("aicc","aic","bic"),restrict=TRUE, allow.multiplicative.trend=FALSE,
     use.initial.values=FALSE, ...)
 {
   #dataname <- substitute(y)
@@ -18,7 +18,7 @@ ets <- function(y, model="ZZZ", damped=NULL,
   y <- as.ts(y)
 
   # Check if data is constant
-  if (is.constant(y)) 
+  if (is.constant(y))
     return(ses(y, alpha=0.99999, initial='simple')$model)
 
   # Remove missing values near ends
@@ -94,6 +94,9 @@ ets <- function(y, model="ZZZ", damped=NULL,
       if(!is.null(lambda))
       {
         model$fitted <- InvBoxCox(model$fitted,lambda)
+        if(biasadj){
+          model$fitted <- InvBoxCoxf(x = model$fitted, fvar = var(model$residuals), lambda = lambda)
+        }
       }
       model$lambda <- lambda
 
@@ -164,7 +167,7 @@ ets <- function(y, model="ZZZ", damped=NULL,
   # Check we have enough data to fit a model
   n <- length(y)
   npars <- 2L # alpha + l0
-  if(trendtype=="A" | trendtype=="M") 
+  if(trendtype=="A" | trendtype=="M")
     npars <- npars + 2L # beta + b0
   if(seasontype=="A" | seasontype=="M")
     npars <- npars + m # gamma + s
@@ -242,6 +245,9 @@ ets <- function(y, model="ZZZ", damped=NULL,
   if(!is.null(lambda))
   {
     model$fitted <- InvBoxCox(model$fitted,lambda)
+    if(biasadj){
+      model$fitted <- InvBoxCoxf(x = model$fitted, fvar = var(model$residuals), lambda = lambda)
+    }
   }
 
   #model$call$data <- dataname
@@ -587,7 +593,7 @@ initparam <- function(alpha,beta,gamma,phi,trendtype,seasontype,damped,lower,upp
     if(m > 12)
       alpha <- 0.0002
     if(is.null(beta) & is.null(gamma))
-      alpha <- lower[1] + .5*(upper[1]-lower[1])
+      alpha <- lower[1] + .1*(upper[1]-lower[1])
     else if(is.null(gamma))
       alpha <- beta+0.001
     else if(is.null(beta))
@@ -1039,3 +1045,7 @@ logLik.ets <- function(object,...)
 {
   structure(object$loglik,df=length(object$par),class="logLik")
 }
+
+is.ets <- function(x){
+  inherits(x, "ets")
+}
\ No newline at end of file
