@@ -1,4 +1,4 @@
-forecast.tbats <- function(object, h, level=c(80,95), fan=FALSE, ...) 
+forecast.tbats <- function(object, h, level=c(80,95), fan=FALSE, biasadj=FALSE, ...) 
 {
 	#Set up variables
   if(missing(h))
@@ -99,7 +99,10 @@ forecast.tbats <- function(object, h, level=c(80,95), fan=FALSE, ...)
 	#Inv Box Cox transform if required
 	if(!is.null(object$lambda))
 	{
-		y.forecast  <-  InvBoxCox(y.forecast,object$lambda)
+	  y.forecast <- InvBoxCox(y.forecast,object$lambda)
+	  if(biasadj){
+	    y.forecast <- InvBoxCoxf(x = list(level = level, mean = y.forecast, upper = upper.bounds, lower = lower.bounds), lambda = object$lambda)
+	  }
 		lower.bounds  <-  InvBoxCox(lower.bounds,object$lambda)
 		if(object$lambda < 1) {
 			lower.bounds<-pmax(lower.bounds, 0)
